/** \file main.c
 *
 * \brief UART: remote echo.
 *
 * This example program receives characters for the console and sends them back.
 *
 * The relevant registers are:
 * - CTRLB: USART Control Register B
 *   - RXEN: Receiver Enable
 *   - TXEN: Transmitter Enable
 * - CTRLC: USART Control Register C
 *   - PMODE: Parity Mode
 *   - SBMODE: Stop Bit Mode
 *   - CHSIZE: Character Size
 * - BAUDCTRLA: USART Baud Rate Register A
 *   - BSEL: Baud Rate
 * - BAUDCTRLB: USART Baud Rate Register B
 *   - BSEL: Baud Rate
 *   - BSCALE: Baud Rate Scale Factor
 * - STATUS: USART Status Register
 *   - RXCIF: Receive Complete Interrupt Flag
 *   - TXCIF: Transmit Complete Interrupt Flag
 * - DATA: USART I/O Data Register
 *
 * Please see modules 4.2.5 and 4.3.5 for more information.
 *
 * \note
 * A console program sends the characters typed on the keyboard to another device and displays the
 * characters received from that device on the screen. We do not see anything when typing at the
 * console as the characters are directly sent to the remote device. Only characters which we
 * receive are displayed. So the remote device has to send the characters back in order to see
 * them. This is called remote echo.
 *
 * \par Baud Rate Table (F_CPU = 2000000)
 *      | Baud   | BSEL | BSCALE |
 *      |-------:|-----:|-------:|
 *      |   9600 |   12 |      0 |
 *      |  14400 |  123 |     -4 |
 *      |  19200 |   11 |     -1 |
 *      |  38400 |    9 |     -2 |
 *      |  57600 |   75 |     -6 |
 *      | 115200 |   11 |     -7 |
 * 
 * \author    Wolfgang Neff
 * \version   1.4
 * \date      2018-06-27
 *
 * \par History
 *      Created: 2013-07-16 \n
 *      Modified: 2014-09-28 \n
 *      Modified: 2015-01-27 \n
 *      Modified: 2017-12-10 \n
 *      Modified: 2018-06-27
 */

#include <avr/io.h>
#include "board.h"

int main(void)
{
	// We need a buffer to store the data received.
	volatile uint8_t buffer;

	/***** Configure the USART pins *****/
	// The TX pin is output and idle at first. Idle means that it must be set high.
	USB_USART_PORT.OUT |= USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR |= USB_USART_TX_PIN_bm;
	// The RX pin is input. We ensure this by clearing DIR although this is default.
	USB_USART_PORT.DIR &= ~USB_USART_RX_PIN_bm;

    /***** Configure the USART module *****/
	// We are using the USB-to-USART gateway. Its configuration is 115200 8N1.
    // Baud rate is 115200 bps. See baud rate table.
    USB_USART_MODULE.BAUDCTRLA = 11;
    USB_USART_MODULE.BAUDCTRLB = (-7<<USART_BSCALE_gp);
	// Line configuration is 8 data bits, no parity, 1 stop bit.
	USB_USART_MODULE.CTRLC = (USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc);
	// Enable receiver and transmitter.
	USB_USART_MODULE.CTRLB = (USART_RXEN_bm | USART_TXEN_bm);

    // We use non-blocking receive and blocking send for the echo.
	while(1)
	{
		// Receive complete? If true the receive complete flag is set.
		if (USB_USART_MODULE.STATUS & USART_RXCIF_bm) {
            // Read data and store it in the buffer.
			buffer = USB_USART_MODULE.DATA;
			// While the data register empty flag is not set do nothing.
			while(!(USB_USART_MODULE.STATUS & USART_DREIF_bm));
			// Now the data register is empty and we can send the data safely.
			USB_USART_MODULE.DATA = buffer;
		}
	}
}
