/** \file main.c
 *
 * \brief ADC: read a sensor.
 *
 * This example program reads the temperature sensor and prints the result
 * of the ADC conversion at the console.
 *
 * The relevant registers are:
 * - CTRLA: Control Register A
 *   - ENABLE: ADC Enable
 * - CTRLB: Control Register B
 *   - RESOLUTION: Conversion Result Resolution
 * - REFCTRL: Reference Control Register
 *   - REFSEL: Reference Selection
 *   - BANDGAP: Bandgap Enable
 * - PRESCALER: Clock Prescaler Register
 *   - PRESCALER: Prescaler Configuration
 * - CHn.RES: Channel n Result Register
 * - CHn.CTRL: Channel n Control Register
 *   - START: Start Conversion
 *   - INPUTMODE: Input Mode
 * - CHn.MUXCTRL: Channel n MUX Control Register
 *   - MUXPOS: MUX Selection on Positive ADC Input
 * - CHn.INTFLAGS: Channel n Interrupt Flag Register
 *   - CHIF: Channel Interrupt Flag
 *
 * The ADC conversion result is calculated by ADC = 4096*Vin/Vref-205.
 * The maximum ADC clock frequency is 2 MHz. So prescaler can be 1.
 *
 * Please see modules 4.2.4 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.3
 * \date      2018-06-27
 *
 * \par History
 *      Created: 2013-07-24 \n
 *      Modified: 2016-10-15 \n
 *      Modified: 2017-12-10 \n
 *      Modified: 2018-06-27
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"
#include "console.h"

// We use the ADC to read the temperature sensor. The temperature sensor has an
// active low enable pin which must be activated. It produces voltages between
// 1.0 V (cold) and 0.0 V (hot). Therefore a reference voltage of 1 V is a good
// choice. 1 V means that the bandgap must be activated. We use channel 0 for
// the conversion and a prescaler division factor of 8.
int main(void)
{
	volatile uint16_t temperature;

	console_init();

	// Initialize temperature sensor.
	SENSOR_PORT.OUT &= ~TEMPERATURE_SENSOR_ENABLE_PIN_bm;
	SENSOR_PORT.DIR |= TEMPERATURE_SENSOR_ENABLE_PIN_bm;
	SENSOR_PORT.DIR &= ~TEMPERATURE_SENSOR_SIGNAL_PIN_bm;

	// Initialize ADC.
	SENSOR_ADC_MODULE.CTRLA = ADC_ENABLE_bm;
	SENSOR_ADC_MODULE.REFCTRL = ADC_REFSEL_INT1V_gc | ADC_BANDGAP_bm;
	SENSOR_ADC_MODULE.PRESCALER = ADC_PRESCALER_DIV8_gc;

	// Configure the ADC channel for temperature sensor.
	SENSOR_ADC_MODULE.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;
	SENSOR_ADC_MODULE.CH0.MUXCTRL = TEMPERATURE_SENSOR_ADC_INPUT;

	while(1)
	{
		// Start conversion.
		SENSOR_ADC_MODULE.CH0.CTRL |= ADC_CH_START_bm;
		// Wait until the conversion has finished.
		while(!(SENSOR_ADC_MODULE.CH0.INTFLAGS & ADC_CH_CHIF_bm));
		// Read the result of the conversion.
		temperature = SENSOR_ADC_MODULE.CH0.RES;
		// Print the result.
		printf("%d\n", temperature);
		// Wait a bit until starting the next conversion.
		_delay_ms(1000);
	}
}
