/** \file main.c
 *
 * \brief I2C: data transfer.
 *
 * This example program sends data via the Inter-Integrated Circuit Interface. The
 * slave has the address 0x20. Pull-up resistors have to be present on the bus.
 *
 * The relevant registers are:
 * - MASTER.BAUD: TWI Baud Rate Register
 * - MASTER.CTRLA: TWI Master Control Register A
 *   - = TWI_MASTER_ENABLE_bm;
 * - MASTER.CTRLC: TWI Master Control Register C
 *   -CMD: Command
 *     - TWI_MASTER_CMD_NOACT_gc: No Action
 *     - TWI_MASTER_CMD_RECVTRANS_gc: Receive or Transmit Data
 *     - TWI_MASTER_CMD_STOP_gc: Issue Stop Condition
 * - MASTER.STATUS: TWI Master Status Register
 *   - WIF: Write Interrupt Flag
 *   - BUSSTATE: Bus State
 * - MASTER.ADDR: Slave Address Register
 * - MASTER.DATA: Data Register
 *
 * The Baud rate is calculated by BAUD = F_CPU/(2*F_I2C)-5 where
 * - F_CPU: System clock frequency
 * - F_I2C: I2C transmission speed
 * The maximum transmission rate is standard mode (100 kbit/s, F_I2C=100000).
 *
 * Please see module 5.3.3 for more information.
 *
 * \note
 * Atmel calls the bus Two-Wire Serial Interface (TWI). So the registers
 * and the bit names start with TWI.
 *
 * \par Bus Speeds (F_CPU = 2000000, TWPS = 0)
 *      | Mode      | Speed      | BAUD | Error |
 *      |-----------|------------|------|-------|
 *      | Low-speed |  10 kbit/s |   95 |    0% |
 *      | Standard  | 100 kbit/s |    5 |    0% |
 *
 * \author    Wolfgang Neff
 * \version   1.3
 * \date      2018-06-27
 *
 * \par History
 *      Created: 2016-10-15 \n
 *      Modified: 2016-11-27 \n
 *      Modified: 2017-12-11 \n
 *      Modified: 2018-06-27
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"

// We define some constants to have better mnemonics (names).
#define I2C_PORT   PORTF
#define I2C_MODULE TWIF
#define I2C_SDA_bm PIN0_bm
#define I2C_SCL_bm PIN1_bm

#define I2C_DATA 'A'
#define I2C_ADDRESS 0x20

// We want to transmit data via the I2C interface on port F. The microcontroller
// is the master of the transmission and sends data to a slave with address 0x20.
// We use the standard transmission speed (100 kbit/s) and ensure that the bus
// state is idle.
int main(void)
{
	/***** Initialize SPI *****/
	// Select standard mode speed.
	I2C_MODULE.MASTER.BAUD = 5;
	// Enable the TWI module.
	I2C_MODULE.MASTER.CTRLA = TWI_MASTER_ENABLE_bm;
	// Force bus state to be idle.
	I2C_MODULE.MASTER.STATUS = TWI_MASTER_BUSSTATE_IDLE_gc;

	while(1)
	{
	    /***** Open write transmission. *****/
		// Send start condition followed by the address with write indicator. 
		I2C_MODULE.MASTER.ADDR = I2C_ADDRESS<<1;
		// Wait until the transmission has finished.
		while (!(I2C_MODULE.MASTER.STATUS & TWI_MASTER_WIF_bm));
		// No action by master necessary.
		I2C_MODULE.MASTER.CTRLC = TWI_MASTER_CMD_NOACT_gc;

	    /***** Send the data. *****/
		// Put the data onto the bus.
		I2C_MODULE.MASTER.DATA = I2C_DATA;
		// Wait until the transmission has finished.
		while (!(I2C_MODULE.MASTER.STATUS & TWI_MASTER_WIF_bm));
		// Finish the transmission.
		I2C_MODULE.MASTER.CTRLC = TWI_MASTER_CMD_RECVTRANS_gc;

	    /***** Close transmission *****/
		// Send stop condition.
		I2C_MODULE.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;

		// Wait a bit before starting the next transmission.
		_delay_ms(1000);
	}
}
